package com.simuwang.base.mapper;

import com.simuwang.base.pojo.dos.AssetDO;
import com.simuwang.base.pojo.vo.FundAssetVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface AssetMapper {

    void batchInsert(@Param("itemDoList") List<AssetDO> assetDOList);

    void batchUpdate(@Param("itemDoList") List<AssetDO> assetDOList);

    List<String> queryFundNavByDate(@Param("fundId") String fundId, @Param("priceDateList") List<String> priceDateList);

    AssetDO queryFundAsset(AssetDO fundAssetVO);

    void deleteAsset(@Param("fundId") String fundId, @Param("priceDate") String priceDate);

    List<AssetDO> selectAssetByFundId(@Param("fundId")String fundId);
}
